@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: cmds.c,v 1.28 1999/07/08 18:59:39 kohl Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -31,7 +35,146 @@
  *
  *	PVM console commands.
  *
-$Log: cmds.c,v $
+ * $Log: cmds.c,v $
+ * Revision 1.28  1999/07/08 18:59:39  kohl
+ * Fixed "Log" keyword placement.
+ * 	- indent with " * " for new CVS.
+ *
+ * Revision 1.27  1999/06/11 17:27:35  pvmsrc
+ * When breaking up multiline commands into subcommands,
+ * 	dump out each command as you execute it, along with a fresh prompt.
+ * (Spanker=kohl)
+ * 
+ * Revision 1.26  1999/06/10 16:57:16  pvmsrc
+ * Fixed docmd() to handle commands with '\n's in them...
+ * 	- a la file input redirection, cut & paste, etc.
+ * 	- break string into individual commands and execute.
+ * (Spanker=kohl)
+ * 
+ * Revision 1.25  1999/02/05  20:36:17  pvmsrc
+ * More contributions from Edward Vigmond <vigmonde@IGB.UMontreal.CA>.
+ * 	- better readline interface, with command completion & help.
+ * (Spanker=kohl)
+ *
+ * Revision 1.24  1998/11/20  19:43:07  pvmsrc
+ * Changes so that win32 will compile & build. Also, common
+ * source for win32 & unix.
+ * (Spanker=sscott)
+ *
+ * Revision 1.23  1998/01/28  23:03:39  pvmsrc
+ * Fixed tracing bogusness!
+ * 	- host add / del notifies for tracing were getting scrunched by
+ * 		console host add notify, etc...
+ * 	- moved message / notify tag constants to job.h and USED THEM.
+ * 	- initialized nextjob properly...
+ * 	- added new "joboffset" global in cons.c for dumping more
+ * 		reasonable job numbers to user.
+ * (Spanker=kohl)
+ *
+ * Revision 1.22  1997/12/31  22:13:15  pvmsrc
+ * Updated "xpvm" trace mask settings.
+ * (Spanker=kohl)
+ *
+ * Revision 1.21  1997/08/29  16:56:53  pvmsrc
+ * Added TEV_USER_DEFINED to "xpvm" default trace mask settings...
+ * (Spanker=kohl)
+ *
+ * Revision 1.20  1997/07/09  13:21:03  pvmsrc
+ * Fixed Author Header.
+ *
+ * Revision 1.19  1997/05/13  14:37:38  pvmsrc
+ * Changed header file $includes:
+ * 	- ../src/listmac.h -> listmac.h
+ * 	- ../src/bfunc.h -> bfunc.h
+ * 	- use -I$(PVMDIR)/src in Makefile.aimk instead.
+ *
+ * Revision 1.18  1997/05/02  20:18:51  pvmsrc
+ * D-Oh.
+ *
+ * Revision 1.17  1997/05/01  15:41:14  pvmsrc
+ * SGI Compiler Warning Cleanup.
+ *
+ * Revision 1.16  1997/04/30  21:15:35  pvmsrc
+ * SGI Compiler Warning Cleanup.
+ *
+ * Revision 1.15  1997/04/17  12:53:15  pvmsrc
+ * rename of pvm_mboxinfo() to include word "get"
+ *
+ * Revision 1.14  1997/04/10  20:34:35  pvmsrc
+ * "CVS": Modified Files:
+ * Typo...
+ *
+ * Revision 1.13  1997/04/10  20:09:33  pvmsrc
+ * Rewrote names_cmd() for new pvm_mboxinfo() syntax.
+ * 	- pvm_getnames() is history.
+ *
+ * Revision 1.12  1997/04/09  20:18:13  pvmsrc
+ * Minor fixes (like being able to actually pass args to "reset"... :-).
+ *
+ * Revision 1.11  1997/04/09  19:53:12  pvmsrc
+ * Merged resetinfo_cmd() command into reset_cmd() routine.
+ * 	- check for args, adjust calling parms to pvmreset()...
+ *
+ * Revision 1.10  1997/04/09  18:31:58  pvmsrc
+ * Added new "resetinfo" command.
+ * 	- like a reset, but without all the task bloodshed...  :-)~
+ *
+ * Revision 1.9  1997/04/08  17:08:46  pvmsrc
+ * Extracted guts of reset_cmd() routine:
+ * 	- created new system pvmreset() routine for general usage.
+ *
+ * Revision 1.8  1997/04/01  19:35:44  pvmsrc
+ * Corrected flag to PvmMbox...
+ *
+ * Revision 1.7  1997/04/01  17:04:45  pvmsrc
+ * Updated mbox stuff:
+ * 	- changed args of pvm_putinfo().
+ * 	- renamed pvm_getinfo() (again) -> pvm_recvinfo(), w/ new syntax.
+ *
+ * Revision 1.6  1997/03/27  20:09:51  pvmsrc
+ * Added PvmNoSpawnParent to pvm_spawn() calls in spawn_cmd().
+ *
+ * Revision 1.5  1997/02/17  16:29:04  pvmsrc
+ * Added checking of pvm_getnoresets() list in reset_cmd().
+ *
+ * Revision 1.4  1997/01/28  19:13:17  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.3  1996/10/24  23:03:39  pvmsrc
+ * Updated to new tracing facilty.
+ * 	- new trace_cmd() options:
+ * 		* "xpvm" mask mimick.
+ * 		* "buf" trace buffer size query & set.
+ * 		* "opt" trace options query & set.
+ * 	- new job creation handling for spawn command,
+ * 		tracing vs. output vs. both...
+ * 	- installed new trc_print_tmask() library call for trace mask dump.
+ *
+ * Revision 1.2  1996/10/08  18:29:39  pvmsrc
+ * Renamed routines:
+ * 	- pvm_put() -> pvm_putinfo().
+ * 	- pvm_get() -> pvm_getinfo().
+ *
+ * Revision 1.1  1996/09/23  20:25:35  pvmsrc
+ * Initial revision
+ *
+ * Revision 1.8  1995/11/02  15:11:40  manchek
+ * added to tickle help
+ *
+ * Revision 1.7  1995/09/05  19:06:52  manchek
+ * help text lowercase
+ *
+ * Revision 1.6  1995/07/03  18:59:51  manchek
+ * help text
+ *
+ * Revision 1.5  1995/07/03  18:57:29  manchek
+ * added ps 's' flag, tickle help 100.
+ * ps doesn't print tasks with tid 0 unless -x flag used
+ *
+ * Revision 1.4  1995/05/17  15:26:17  manchek
+ * added "return 0" to lots of commands.
+ * added idump command
+ *
  * Revision 1.3  1994/10/15  18:38:56  manchek
  * updated task flags and ps list headers
  *
@@ -56,37 +199,51 @@ $Log: cmds.c,v $
 #include <pvmtev.h>
 #include "cmd.h"
 #include "myalloc.h"
-#include "../src/listmac.h"
-#include "../src/bfunc.h"
+#include "listmac.h"
+#include "bfunc.h"
 #include "job.h"
-#include "tevfmt.h"
 
 #define	PVMERRMSG(n)	((n) <= 0 && (n) > -pvm_nerr \
 						? pvm_errlist[-(n)] : "Unknown Error")
 
-extern char *getenv();
+char *getenv();
+void i_dump();
 
 extern char **environ;
 
 extern char *pvm_errlist[];
 extern int pvm_nerr;
 
+extern struct Pvmtevinfo pvmtevinfo[];
+
+extern char *prompt;
+
 int add_cmd();
 int alias_cmd();
 int conf_cmd();
 int delete_cmd();
 int echo_cmd();
+int export_cmd();
 int halt_cmd();
 int help_cmd();
 int id_cmd();
+int idump_cmd();
+/*
 int insert_cmd();
+*/
 int jobs_cmd();
 int kill_cmd();
+/*
 int lookup_cmd();
+*/
 int mstat_cmd();
+int names_cmd();
 int quit_cmd();
 int pstat_cmd();
+int put_cmd();
+/*
 int remove_cmd();
+*/
 int reset_cmd();
 int setenv_cmd();
 int sig_cmd();
@@ -96,6 +253,7 @@ int tasks_cmd();
 int tickle_cmd();
 int trace_cmd();
 int unalias_cmd();
+int unexport_cmd();
 int version_cmd();
 
 extern struct alias *aliases;			/* from cons.c */
@@ -103,9 +261,9 @@ extern struct pvmhostinfo *hostlist;	/* from cons.c */
 extern struct job *joblist;				/* from job.c */
 extern int mytid;						/* from cons.c */
 extern int narchs;						/* from cons.c */
+extern int joboffset;					/* from cons.c */
 extern int nextjob;						/* from cons.c */
 extern int nhosts;						/* from cons.c */
-extern struct tevfmt tev_formats[];		/* from tevfmt.c */
 
 
 struct cmdsw commands[] = {
@@ -114,31 +272,40 @@ struct cmdsw commands[] = {
 	{ "conf",    1, 1, conf_cmd },
 	{ "delete",  2, 0, delete_cmd },
 	{ "echo",    1, 0, echo_cmd },
+	{ "export",    1, 0, export_cmd },
 	{ "halt",    1, 1, halt_cmd },
 	{ "help",    1, 2, help_cmd },
 	{ "id",      1, 1, id_cmd },
+	{ "idump",   1, 2, idump_cmd },
+/*
 	{ "insert",  4, 4, insert_cmd },
+*/
 	{ "jobs",    1, 2, jobs_cmd },
 	{ "kill",    2, 0, kill_cmd },
+/*
 	{ "lookup",  3, 3, lookup_cmd },
+*/
 	{ "mstat",   2, 0, mstat_cmd },
+	{ "names",   1, 2, names_cmd },
 	{ "ps",      1, 2, tasks_cmd },
 	{ "pstat",   2, 0, pstat_cmd },
+	{ "put",     4, 4, put_cmd },
 	{ "quit",    1, 1, quit_cmd },
+/*
 	{ "remove",  3, 3, remove_cmd },
-	{ "reset",   1, 1, reset_cmd },
+*/
+	{ "reset",   1, 4, reset_cmd },
 	{ "setenv",  1, 0, setenv_cmd },
 	{ "sig",     3, 0, sig_cmd },
 	{ "spawn",   2, 0, spawn_cmd },
 	{ "trace",   1, 0, trace_cmd },
 	{ "tickle",  2, 11, tickle_cmd },
 	{ "unalias", 2, 0, unalias_cmd },
+	{ "unexport", 1, 0, unexport_cmd },
 	{ "version", 1, 1, version_cmd },
 	{ 0, 0, 0, 0 }
 };
 
-static char rcsid[] = "$Id: cmds.c,v 1.3 1994/10/15 18:38:56 manchek Exp $";
-
 
 freealias(ap)
 	struct alias *ap;
@@ -198,65 +365,100 @@ docmd(cmd)
 	int ac;
 	char *av[128];
 	int self;
+	char *nextcmd = NULL;
+	int first = 1;
 
 	/*
-	* parse command
+	* check for '\n's in command, separate commands
 	*/
 
-	ac = sizeof(av)/sizeof(av[0]) - 1;
-	if (acav(cmd, &ac, av)) {
-		fputs("command too long\n", stdout);
-		return 0;
-	}
-	if (!ac)
-		return 0;
+	nextcmd = cmd;
 
-	/*
-	* resolve aliases
-	*/
+	do
+	{
+		/*
+		* look for a newline...
+		*/
 
-	for (ap = aliases->a_link; ap != aliases; ap = ap->a_link)
-		ap->a_flag = 0;
+		cmd = nextcmd;
 
-	while (ap = findalias(av[0])) {
-		if (ap->a_flag) {
-			printf("alias loop\n");
+		while ( *nextcmd != '\0' && *nextcmd != '\n' )
+			nextcmd++;
+
+		if ( *nextcmd == '\n' )
+		{
+			*nextcmd++ = '\0';
+
+			if ( !first )
+				printf( prompt );
+			
+			else
+				first = 0;
+
+			printf( "%s\n", cmd );
+		}
+
+		/*
+		* parse command
+		*/
+
+		ac = sizeof(av)/sizeof(av[0]) - 1;
+		if (acav(cmd, &ac, av)) {
+			fputs("command too long\n", stdout);
 			return 0;
 		}
-		ap->a_flag = 1;
-		if (ap->a_num > 1) {
-			if (ac + ap->a_num > sizeof(av)/sizeof(av[0])) {
-				fputs("command too long\n", stdout);
+		if (!ac)
+			return 0;
+
+		/*
+		* resolve aliases
+		*/
+
+		for (ap = aliases->a_link; ap != aliases; ap = ap->a_link)
+			ap->a_flag = 0;
+
+		while (ap = findalias(av[0])) {
+			if (ap->a_flag) {
+				printf("alias loop\n");
 				return 0;
 			}
-			for (i = ac; --i > 0; )
-				av[i + ap->a_num - 1] = av[i];
-			ac += ap->a_num - 1;
+			ap->a_flag = 1;
+			if (ap->a_num > 1) {
+				if (ac + ap->a_num > sizeof(av)/sizeof(av[0])) {
+					fputs("command too long\n", stdout);
+					return 0;
+				}
+				for (i = ac; --i > 0; )
+					av[i + ap->a_num - 1] = av[i];
+				ac += ap->a_num - 1;
+			}
+			self = !strcmp(av[0], ap->a_args[0]);
+			for (i = ap->a_num; i-- > 0; )
+				av[i] = ap->a_args[i];
+			if (self)
+				break;
 		}
-		self = !strcmp(av[0], ap->a_args[0]);
-		for (i = ap->a_num; i-- > 0; )
-			av[i] = ap->a_args[i];
-		if (self)
-			break;
-	}
 
-	/*
-	* find command and call it
-	*/
+		/*
+		* find command and call it
+		*/
 
-	av[ac] = 0;
-	p = av[0];
-	for (csp = commands; csp->cmd; csp++) {
-		if (!strcmp(csp->cmd, p)) {
-			if (ac >= csp->a1 && (ac <= csp->a2 || !csp->a2))
-				(csp->fun)(ac, av);
-			else
-				printf("%s: wrong #args\n", p);
-			break;
+		av[ac] = 0;
+		p = av[0];
+		for (csp = commands; csp->cmd; csp++) {
+			if (!strcmp(csp->cmd, p)) {
+				if (ac >= csp->a1 && (ac <= csp->a2 || !csp->a2))
+					(csp->fun)(ac, av);
+				else
+					printf("%s: wrong #args\n", p);
+				break;
+			}
 		}
+		if (!csp->cmd)
+			printf("%s: not found\n", p);
 	}
-	if (!csp->cmd)
-		printf("%s: not found\n", p);
+	while ( *nextcmd != '\0' );
+
 	return 0;
 }
 
@@ -279,7 +481,7 @@ add_cmd(ac, av)
 	sv = TALLOC(ac, int, "int");
 	if ((cc = pvm_addhosts(av, ac, sv)) >= 0) {
 		if (cc > 0)
-			pvm_recv(-1, 0);	/* waste the notify message */
+			pvm_recv(-1, HostsAddedTag); /* waste the notify message */
 
 		printf("%d successful\n", cc);
 		fputs("                    HOST     DTID\n", stdout);
@@ -319,6 +521,7 @@ alias_cmd(ac, av)
 			freealias(ap);
 		newalias(av[1], ac - 2, av + 2);
 	}
+	return 0;
 }
 
 
@@ -333,14 +536,16 @@ conf_cmd(ac, av)
 	if (!pvm_config(&nhosts, &narchs, &hostlist)) {
 		printf("%d host%s, %d data format%s\n",
 			nhosts, (nhosts > 1 ? "s" : ""), narchs, (narchs > 1 ? "s" : ""));
-		fputs("                    HOST     DTID     ARCH   SPEED\n", stdout);
+		fputs("                    HOST     DTID     ARCH   SPEED       DSIG\n", stdout);
 		for (i = 0; i < nhosts; i++)
-			printf("%24s %8x %8s%8d\n",
+			printf("%24s %8x %8s%8d 0x%08x\n",
 					hostlist[i].hi_name,
 					hostlist[i].hi_tid,
 					hostlist[i].hi_arch,
-					hostlist[i].hi_speed);
+					hostlist[i].hi_speed,
+					hostlist[i].hi_dsig);
 	}
+	return 0;
 }
 
 
@@ -367,6 +572,7 @@ delete_cmd(ac, av)
 		}
 	}
 	MY_FREE(sv);
+	return 0;
 }
 
 
@@ -396,18 +602,19 @@ halt_cmd(ac, av)
 		pvmendtask();
 		exit(0);
 	}
+	return 0;
 }
 
 
-static char *helptx[] = {
+char *helptx[] = {
 
-	"add ADD    - Add hosts to virtual machine",
+	"add add         Add hosts to virtual machine",
 	"add Syntax:  add hostname ...",
 
-	"alias ALIAS  - Define/list command aliases",
+	"alias alias       Define/list command aliases",
 	"alias Syntax:  alias [name command args ...]",
 
-	"conf CONF   - List virtual machine configuration",
+	"conf conf        List virtual machine configuration",
 	"conf Syntax:  conf",
 	"conf Output fields:",
 	"conf   HOST    host name",
@@ -415,80 +622,101 @@ static char *helptx[] = {
 	"conf   ARCH    xhost architecture",
 	"conf   SPEED   host relative speed",
 
-	"delete DELETE - Delete hosts from virtual machine",
+	"delete delete      Delete hosts from virtual machine",
 	"delete Syntax:  delete hostname ...",
 
-	"echo ECHO   - Echo arguments",
+	"echo echo        Echo arguments",
 	"echo Syntax:  echo [ arg ... ]",
 
-	"halt HALT   - Stop pvmds",
+	"export export      Add environment variables to spawn export list",
+	"export Syntax:  export [ varname ... ]",
+
+	"halt halt        Stop pvmds",
 	"halt Syntax:  halt",
 
-	"help HELP   - Print helpful information about a command",
+	"help help        Print helpful information about a command",
 	"help Syntax:  help [ command ]",
 
-	"id ID     - Print console task id",
+	"id id          Print console task id",
 	"id Syntax:  id",
 
-	"insert-INSERT - Add entry to database",
-	"insert-Syntax:  insert name index|-1 hexvalue",
+	"idump-idump          Call i_dump to display heap contents",
+	"idump-Syntax:  idump [ how ]",
 
-	"jobs JOBS   - Display list of running jobs",
+/*
+	"insert-insert      Add entry to database",
+	"insert-Syntax:  insert name index|-1 hexvalue",
+*/
+
+	"jobs jobs        Display list of running jobs",
 	"jobs Syntax:  jobs [ options ]",
 	"jobs Options:  -l   give long listing",
 
-	"kill KILL   - Terminate tasks",
+	"kill kill        Terminate tasks",
 	"kill Syntax:  kill [ options ] tid ...",
 	"kill Options:  -c   kill children of tid",
 
-	"lookup-LOOKUP - Find entry in database",
+/*
+	"lookup-lookup      Find entry in database",
 	"lookup-Syntax:  lookup name index|-1",
+*/
 
-	"mstat MSTAT  - Show status of hosts",
+	"names names       List message mailbox names",
+	"names Syntax:  names [ pattern ]",
+
+	"mstat mstat       Show status of hosts",
 	"mstat Syntax:  mstat name ...",
 
-	"ps PS     - List tasks",
+	"ps ps          List tasks",
 	"ps Syntax:  ps [ -axh ]",
 	"ps Options:  -a       all hosts (default is local)",
 	"ps           -hhost   specific host tid",
 	"ps           -nhost   specific host name",
 	"ps           -l       long (show process id)",
-	"ps           -x       show console task (default is not)",
+	"ps           -x       show all tasks (e.g. console and nulls)",
 	"ps Output fields:",
 	"ps   HOST    host name",
-	"ps   A.OUT   executable name",
 	"ps   TID     task id",
 	"ps   PTID    parent task id",
-	"ps   PTID    task process id",
+	"ps   PID     task process id",
 	"ps   FLAG    status",
+	"ps   COMMAND executable name",
 	"ps FLAG values:",
 	"ps   f   task process is child of pvmd",
 	"ps   c   task connected to pvmd",
 	"ps   a   task waiting authorization",
 	"ps   o   task connection being closed",
+	"ps   s   task needs too many shared pages, is deadlocked",
 	"ps   H   hoster task",
 	"ps   R   resource manager task",
 	"ps   T   tasker task",
 
-	"pstat PSTAT  - Show status of tasks",
+	"pstat pstat       Show status of tasks",
 	"pstat Syntax:  pstat tid ...",
 
-	"quit QUIT   - Exit console",
+	"put put         Add entry to message mailbox",
+	"put Syntax:  put name flags",
+
+	"quit quit        Exit console",
 	"quit Syntax:  quit",
 
-	"remove-REMOVE - Delete entry from database",
+/*
+	"remove-remove      Delete entry from database",
 	"remove-Syntax:  remove name index",
+*/
 
-	"reset RESET  - Kill all tasks",
-	"reset Syntax:  reset",
+	"reset reset       Kill all tasks, delete leftover mboxes",
+	"reset Syntax:  reset\n",
+	"reset reset info    Delete orphaned persistent mbox entries",
+	"reset Syntax:  reset info [ class [ index ] ]",
 
-	"setenv SETENV - Display or set environment variables",
+	"setenv setenv      Display or set environment variables",
 	"setenv Syntax:  setenv [ name [ value ] ]",
 
-	"sig SIG    - Send signal to task",
+	"sig sig         Send signal to task",
 	"sig Syntax:  sig signum task ...",
 
-	"spawn SPAWN  - Spawn task",
+	"spawn spawn       Spawn task",
 	"spawn Syntax:  spawn [ options ] file [ arg ... ]",
 	"spawn Options:  -(count)  number of tasks, default is 1",
 	"spawn           -(host)   spawn on host, default is any",
@@ -500,31 +728,55 @@ static char *helptx[] = {
 	"spawn           -@        trace job, display output on console",
 	"spawn           -@(file)  trace job, output to file",
 
-	"tickle-TICKLE - Tickle pvmd",
+	"tickle-tickle      Tickle pvmd",
 	"tickle-Syntax:  tickle how [ arg ... ]",
 	"tickle-How:",
 	"tickle-  0   dump heap",
 	"tickle-  1   dump host table",
 	"tickle-  2   dump local task table",
 	"tickle-  3   dump waitc list",
-	"tickle-  4   dump class-name list",
+	"tickle-  4   dump message mailbox",
 	"tickle-  5   get debugmask",
 	"tickle-  6   (mask) set debugmask",
+	"tickle-        mask is the sum of the following bits for information about",
+	"tickle-           1  Packet routing",
+	"tickle-           2  Message routing and entry points",
+	"tickle-           4  Task state",
+	"tickle-           8  Slave pvmd startup",
+	"tickle-          16  Host table updates",
+	"tickle-          32  Select loop",
+	"tickle-          64  IP network",
+	"tickle-         128  Multiprocessor nodes",
+	"tickle-         256  Resource manager interface",
+	"tickle-         512  Application warnings (scrapped messages etc.)",
+	"tickle-        1024  Wait contexts",
+	"tickle-        2048  Shared memory operations",
+	"tickle-        4096  Semaphores",
+	"tickle-        8192  Locks",
 	"tickle-  7   (num) set nopax",
 	"tickle-  8   (dtid) trigger hostfail",
 	"tickle-  9   (rst) dump pvmd statistics, clear if rst true",
+	"tickle-  100 dump shared memory data structures",
 
-	"trace TRACE  - Set/display trace event mask",
+	"trace trace       Set/display trace event mask",
 	"trace Syntax:  trace",
 	"trace          trace [+] name ...",
 	"trace          trace - name ...",
 	"trace          trace [+] *",
 	"trace          trace - *",
+	"trace          trace xpvm (default mask for XPVM display)",
+	"trace          trace buf nbytes",
+	"trace          trace buf",
+	"trace          trace opt < full | time | count >",
+	"trace          trace opt",
 
-	"unalias UNALIAS - Undefine command alias",
+	"unalias unalias     Undefine command alias",
 	"unalias Syntax:  unalias name ...",
 
-	"version VERSION - Show libpvm version",
+	"unexport unexport    Remove environment variables from spawn export list",
+	"unexport Syntax:  unexport [ varname ... ]",
+
+	"version version     Show libpvm version",
 	"version Syntax:  version",
 	0
 };
@@ -575,9 +827,24 @@ id_cmd(ac, av)
 	ac = ac;
 	av = av;
 	printf("t%x\n", mytid);
+	return 0;
 }
 
 
+idump_cmd(ac, av)
+	int ac;
+	char **av;
+{
+	int how = 0;
+
+	if (ac > 1)
+		how = atoi(av[1]);
+	i_dump(how);
+	return 0;
+}
+
+
+/*
 insert_cmd(ac, av)
 	int ac;
 	char **av;
@@ -592,7 +859,9 @@ insert_cmd(ac, av)
 	else
 		if (cc == PvmDupEntry)
 			printf("already exists\n");
+	return 0;
 }
+*/
 
 
 jobs_cmd(ac, av)
@@ -614,7 +883,7 @@ jobs_cmd(ac, av)
 
 	for (jp = joblist->j_link; jp != joblist; jp = jp->j_link) {
 		printf("%c%d:%s", (jp->j_flag & JOB_TRACE ? 'T' : ' '),
-				jp->j_jid, (l ? "\n" : ""));
+				jp->j_jid - joboffset, (l ? "\n" : ""));
 		if (jp->j_obufs) {
 			for (op = jp->j_obufs->o_link; op != jp->j_obufs; op = op->o_link)
 				if (l) {
@@ -665,7 +934,7 @@ kill_cmd(ac, av)
 
 	if (ac < 2) {
 		fputs("incorrect arg count\n", stdout);
-		return;
+		return 1;
 	}
 
 	if (cflg && pvm_tasks(host, &ntask, &tip) < 0)
@@ -690,9 +959,11 @@ kill_cmd(ac, av)
 				pvm_kill(tid);
 		}
 	}
+	return 0;
 }
 
 
+/*
 lookup_cmd(ac, av)
 	int ac;
 	char **av;
@@ -705,7 +976,9 @@ lookup_cmd(ac, av)
 	else
 		if (cc == PvmNoEntry)
 			printf("no such entry\n");
+	return 0;
 }
+*/
 
 
 mstat_cmd(ac, av)
@@ -719,6 +992,7 @@ mstat_cmd(ac, av)
 		cc = pvm_mstat(av[i]);
 		printf("%24s  %s\n", av[i], (cc < 0 ? PVMERRMSG(cc) : "ok"));
 	}
+	return 0;
 }
 
 
@@ -735,6 +1009,7 @@ pstat_cmd(ac, av)
 		cc = pvm_pstat(tid);
 		printf("t%8x  %s\n", tid, (cc < 0 ? PVMERRMSG(cc) : "run"));
 	}
+	return 0;
 }
 
 
@@ -745,10 +1020,15 @@ quit_cmd()
 		pvm_exit();
 		printf("pvmd still running.\n");
 	}
+#ifdef WIN32
+	WSACleanup();
+#endif
 	exit(0);
+	return 0;  /* for stupid SGI compiler :-Q */
 }
 
 
+/*
 remove_cmd(ac, av)
 	int ac;
 	char **av;
@@ -761,28 +1041,40 @@ remove_cmd(ac, av)
 	else
 		if (cc == PvmNoEntry)
 			printf("no such entry\n");
+	return 0;
 }
+*/
 
 
 reset_cmd(ac, av)
 	int ac;
 	char **av;
 {
-	struct pvmtaskinfo *tip;
-	int ntask;
-	int i;
-	int j;
+	char *class = (char *) NULL;
 
-	ac = ac;
-	av = av;
-	if (!pvm_tasks(0, &ntask, &tip))
-		for (i = 0; i < ntask; i++)
-			if (tip[i].ti_tid && tip[i].ti_tid != mytid)
-				pvm_kill(tip[i].ti_tid);
+	int killtasks = 1;
+	int index = -1;
 
-	/* XXX this is gnasty... */
-	while ((i = pvm_lookup("pvmgs", -1, &j)) >= 0)
-		pvm_delete("pvmgs", i);
+	/* Check for Info (mbox) Reset */
+
+	if ( ac > 1 ) {
+		if ( !strcmp( av[1], "info" ) ) {
+			killtasks = 0;
+			if ( ac > 2 ) {
+				class = av[2];
+				if ( ac > 3 )
+					index = atoi( av[3] );
+			}
+		} else {
+			printf( "\"%s\" arg to reset not valid\n", av[1] );
+			return 1;
+		}
+	}
+
+	/* Call the universal VM-spanker routine...  :-) */
+	pvmreset( mytid, killtasks, class, index );
+
+	return 0;
 }
 
 
@@ -843,6 +1135,7 @@ sig_cmd(ac, av)
 		tid = tidtoi(av[i]);
 		pvm_sendsig(tid, signum);
 	}
+	return 0;
 }
 
 
@@ -852,7 +1145,7 @@ spawn_cmd(ac, av)
 {
 	int *tids = 0;
 	char *where = 0;
-	int flags = 0;
+	int flags = PvmNoSpawnParent;
 	int count = 1;
 	int i;
 	int oflg = 0;
@@ -861,6 +1154,9 @@ spawn_cmd(ac, av)
 	char *ofn = 0;
 	char *tfn = 0;
 	struct job *jp, *jp2;
+	void status_msg();
+	void event_dump_hdr();
+	void output_dump_hdr();
 
 	while (av[1][0] == '-') {
 		if (ac < 3) {
@@ -902,44 +1198,63 @@ spawn_cmd(ac, av)
 			tflg = 1;
 			tfn = av[1] + 2;
 			if (!*tfn)
-				tfn = 0;
+				tfn = "";
 		}
 		av++;
 		ac--;
 	}
-	if (oflg) {
-		pvm_setopt(PvmOutputTid, mytid);
-		pvm_setopt(PvmOutputCode, nextjob);
-		jp = job_new(nextjob);
-		printf("[%d]\n", nextjob++);
-		if (ofn) {
-			jp->j_ff = fopen(ofn, (app ? "a" : "w"));
-			if (!jp->j_ff) {
-				perror(ofn);
-				job_free(jp);
-				return 1;
-			}
-		}
-	} else
-		pvm_setopt(PvmOutputTid, 0);
-
 	if (tflg) {
-		pvm_setopt(PvmTraceTid, mytid);
-		pvm_setopt(PvmTraceCode, nextjob);
 		jp2 = job_new(nextjob);
 		jp2->j_flag |= JOB_TRACE;
-		printf("[%d]\n", nextjob++);
-		if (tfn) {
-			jp2->j_ff = fopen(tfn, "w");
-			if (!jp2->j_ff) {
-				perror(tfn);
+		jp2->j_trcid = trc_get_tracer_id();
+		jp2->j_trcid->status_msg = status_msg;
+		if (tfn && !strcmp(tfn,""))
+			jp2->j_trcid->event_dump_hdr = event_dump_hdr;
+		if (oflg && !ofn) ofn = "";
+		if (ofn)
+			jp2->j_trcid->output_dump_hdr = output_dump_hdr;
+		jp2->j_trcid->event_ctx = pvm_getcontext();
+		jp2->j_trcid->event_tag = nextjob;
+		jp2->j_trcid->output_ctx = pvm_getcontext();
+		jp2->j_trcid->output_tag = nextjob + 1;
+		trc_set_tracing_codes( jp2->j_trcid );
+		printf("[%d]\n", nextjob - joboffset);
+		nextjob += 2;
+		trc_set_trace_file( jp2->j_trcid, tfn );
+		if (!trc_reset_trace_file( jp2->j_trcid )) {
+			job_free(jp2);
+			return 1;
+		}
+		trc_save_host_status_events( jp2->j_trcid );
+		if (ofn) {
+			trc_set_output_file( jp2->j_trcid, ofn );
+			if (!trc_open_output_file( jp2->j_trcid )) {
 				job_free(jp2);
 				return 1;
 			}
 		}
-
-	} else {
+		oflg = 0;
+	}
+	else {
 		pvm_setopt(PvmTraceTid, 0);
+
+		if (oflg) {
+			pvm_setopt(PvmOutputTid, mytid);
+			pvm_setopt(PvmOutputContext, pvm_getcontext());
+			pvm_setopt(PvmOutputCode, nextjob);
+			jp = job_new(nextjob);
+			printf("[%d]\n", nextjob - joboffset);
+			nextjob++;
+			if (ofn) {
+				jp->j_ff = fopen(ofn, (app ? "a" : "w"));
+				if (!jp->j_ff) {
+					perror(ofn);
+					job_free(jp);
+					return 1;
+				}
+			}
+		} else
+			pvm_setopt(PvmOutputTid, 0);
 	}
 
 	tids = TALLOC(count > 1 ? count : 1, int, "int");
@@ -961,7 +1276,7 @@ spawn_cmd(ac, av)
 
 
 static char *tflgs[] = {
-	0, "f", "c", "a", "o", 0, 0, 0,
+	0, "f", "c", "a", "o", "s", 0, 0,
 	"R", "H", "T"
 };
 
@@ -997,6 +1312,7 @@ print_task_hdr(lflg)
 	else
 		fputs("                    HOST      TID   FLAG 0x COMMAND\n",
 			stdout);
+	return 0;
 }
 
 
@@ -1106,12 +1422,13 @@ tasks_cmd(ac, av)
 	if (!pvm_tasks(host, &ntask, &tip)) {
 		print_task_hdr(lflg);
 		for (i = 0; i < ntask; i++) {
-			if (tip[i].ti_tid == mytid && !xflg)
+			if (!xflg && (tip[i].ti_tid == mytid || tip[i].ti_tid == 0))
 				continue;
 			print_task_rec(&tip[i], lflg);
 			printf("\n");
 		}
 	}
+	return 0;
 }
 
 
@@ -1138,66 +1455,16 @@ tickle_cmd(ac, av)
 }
 
 
-int sortedtevs[] = {
-	0,   154, 2,   4,   6,
-	156, 8,   10,  12,  14,  16,  178, 18,
-	20,  158, 22,  24,  26,  28,  160, 30,
-	32,  162, 34,  36,  38,  40,  42,  44,
-	46,  48,  50,  52,  54,  56,  164, 58,
-	60,  62,  64,  66,  68,  70,  72,  76,
-	80,  84,  74,  78,  82,  166, 86,  168,
-	88,  90,  92,  170, 142, 144, 146, 180,
-	94,  96,  172, 98,  100, 102, 174, 104,
-	106, 108, 110, 112, 114, 176, 116, 118,
-	120, 122, 124, 126, 130, 134, 138, 128,
-	132, 136, 140, 150, 148, 152,
-};
-
 printtm(who)
 	int who;
 {
-	int maxt = sizeof(sortedtevs)/sizeof(sortedtevs[0]);
-/*
-	int maxt = TEV_MAX / 2 + 1;
-*/
-	int h[4];
-	int i[3];
-	int j;
-	int b;
-	int d;
-	int l;
-	char buf[64];
 	Pvmtmask tm;
 
 	pvm_gettmask(who, tm);
 
-	i[0] = h[0] = 0;
-	i[1] = h[1] = maxt / 3;
-	i[2] = h[2] = (maxt * 2) / 3;
-	h[3] = maxt;
+	trc_print_tmask(tm);
 
-	do {
-		d = 0;
-		for (j = 0; j < 3; j++) {
-			if (i[j] < h[j + 1]) {
-				b = TEV_CHECK_MASK(tm, sortedtevs[i[j]]);
-				strcpy(buf, tev_formats[sortedtevs[i[j]]].name);
-/*
-				b = TEV_CHECK_MASK(tm, i[j] * 2);
-				strcpy(buf, tev_formats[i[j] * 2].name);
-*/
-				l = strlen(buf) - 1;
-				if (buf[l] == '0')
-					buf[l] = 0;
-				printf(" %c %-22s", (b ? '*' : ' '), buf);
-				i[j]++;
-			} else
-				printf("                         ");
-			if (i[j] >= h[j + 1])
-				d++;
-		}
-		printf("\n");
-	} while (d != 3);
+	return 0;
 }
 
 
@@ -1205,12 +1472,16 @@ trace_cmd(ac, av)
 	int ac;
 	char **av;
 {
+	static int topt = PvmTraceFull;
+	static int tbuf = 0;
 	int i;
 	int onoff = 1;
 	int e;
 	int l;
 	char buf[64];
+	char *optstr;
 	Pvmtmask tm;
+	int settm;
 
 	if (ac == 1) {
 		printtm(PvmTaskChild);
@@ -1219,44 +1490,105 @@ trace_cmd(ac, av)
 
 	pvm_gettmask(PvmTaskChild, tm);
 
+	settm = 0;
+
 	for (i = 1; i < ac; i++) {
 		if (!strcmp(av[i], "+")) {
 			onoff = 1;
 		} else if (!strcmp(av[i], "-")) {
 			onoff = 0;
 		} else if (!strcmp(av[i], "*")) {
-			for (e = TEV_FIRST; e <= TEV_MAX; e += 2)
+			for (e = TEV_FIRST; e <= TEV_MAX; e++) {
 				if (onoff)
-					TEV_SET_MASK(tm, e);
+					TEV_MASK_SET(tm, e);
 				else
-					TEV_UNSET_MASK(tm, e);
-		} else {
-			if (strncmp(av[i], "pvm_", 4))
-				strcpy(buf, "pvm_");
-			else
-				buf[0] = 0;
-			strcat(buf, av[i]);
-			l = strlen(buf);
-/*
-			printf("<%s>\n", buf);
-*/
-			for (e = TEV_FIRST; e <= TEV_MAX; e += 2)
-				if (!strncmp(tev_formats[e].name, buf, l)
-				|| !strncmp(tev_formats[e].name, av[i], l))
-					break;
-			if (e <= TEV_MAX) {
-				if (onoff)
-					TEV_SET_MASK(tm, e);
-				else
-					TEV_UNSET_MASK(tm, e);
-
-			} else {
-				printf("no such event \"%s\"\n", av[i]);
+					TEV_MASK_UNSET(tm, e);
 			}
+			settm++;
+		} else if (!strcmp(av[i], "buf")) {
+			if ( i + 1 < ac ) {
+				tbuf = atoi(av[++i]);
+				pvm_setopt( PvmTraceBuffer, tbuf );
+			} else {
+				printf("\nTrace Buffering set to %d bytes\n\n", tbuf);
+			}
+		} else if (!strcmp(av[i], "opt")) {
+			if ( i + 1 < ac ) {
+				i++;
+				if ( !strcmp( av[i], "full" ) )
+					topt = PvmTraceFull;
+				else if ( !strcmp( av[i], "time" ) )
+					topt = PvmTraceTime;
+				else if ( !strcmp( av[i], "count" ) )
+					topt = PvmTraceCount;
+				else {
+					printf( "\nError:  Unknown Trace Option \"%s\"\n\n",
+						av[i] );
+				}
+				pvm_setopt( PvmTraceOptions, topt );
+			} else {
+				switch ( topt )
+				{
+					case PvmTraceFull:		optstr = "full";	break;
+					case PvmTraceTime:		optstr = "time";	break;
+					case PvmTraceCount:		optstr = "count";	break;
+					default: {
+						sprintf( buf, "<Unknown=%d>", topt );
+						optstr = buf;
+					}
+				}
+				printf("\nTrace Options set to \"%s\"\n\n", optstr );
+			}
+		} else if (!strcmp(av[i], "xpvm")) {
+			TEV_MASK_INIT( tm );
+			TEV_MASK_SET( tm, TEV_BARRIER );
+			TEV_MASK_SET( tm, TEV_BCAST );
+			TEV_MASK_SET( tm, TEV_JOINGROUP );
+			TEV_MASK_SET( tm, TEV_LVGROUP );
+			TEV_MASK_SET( tm, TEV_MYTID );
+			TEV_MASK_SET( tm, TEV_PARENT );
+			TEV_MASK_SET( tm, TEV_SIBLINGS );
+			TEV_MASK_SET( tm, TEV_SPAWN );
+			TEV_MASK_SET( tm, TEV_SENDSIG );
+			TEV_MASK_SET( tm, TEV_KILL );
+			TEV_MASK_SET( tm, TEV_EXIT );
+			TEV_MASK_SET( tm, TEV_MCAST );
+			TEV_MASK_SET( tm, TEV_SEND );
+			TEV_MASK_SET( tm, TEV_RECV );
+			TEV_MASK_SET( tm, TEV_RECVF );
+			TEV_MASK_SET( tm, TEV_NRECV );
+			TEV_MASK_SET( tm, TEV_PSEND );
+			TEV_MASK_SET( tm, TEV_PRECV );
+			TEV_MASK_SET( tm, TEV_TRECV );
+			TEV_MASK_SET( tm, TEV_PROBE );
+			TEV_MASK_SET( tm, TEV_ADDMHF );
+			TEV_MASK_SET( tm, TEV_DELMHF );
+			TEV_MASK_SET( tm, TEV_MHF_INVOKE );
+			TEV_MASK_SET( tm, TEV_USER_DEFINED );
+			settm++;
+		} else {
+			if (!strncmp(av[i], "pvm_", 4))
+				strcpy(buf, av[i] + 4);
+			else
+				strcpy(buf, av[i]);
+			for (e = TEV_FIRST; e <= TEV_MAX; e++)
+				if (!strcmp(pvmtevinfo[e].name, buf)
+				|| !strcmp(pvmtevinfo[e].name, av[i]))
+					break;
+			if ( e <= TEV_MAX ) {
+				if (onoff)
+					TEV_MASK_SET(tm, e);
+				else
+					TEV_MASK_UNSET(tm, e);
+				settm++;
+			}
+			else
+				printf("no such event \"%s\"\n", av[i]);
 		}
 	}
 
-	pvm_settmask(PvmTaskChild, tm);
+	if ( settm )
+		pvm_settmask(PvmTaskChild, tm);
 
 	return 0;
 }
@@ -1287,3 +1619,72 @@ version_cmd(ac, av)
 }
 
 
+names_cmd(ac, av)
+	int ac;
+	char **av;
+{
+	struct pvmmboxinfo *classes;
+	int nclasses;
+	int i, j;
+	int cc;
+
+	pvm_getmboxinfo((ac > 1 ? av[1] : "*"), &nclasses, &classes);
+	for ( i=0 ; i < nclasses ; i++ )
+	{
+		printf("<%s>:\n", classes[i].mi_name);
+		for ( j=0 ; j < classes[i].mi_nentries ; j++ )
+			printf("\tindex=%d owner=0x%x/%d flags=0x%x\n",
+				classes[i].mi_indices[j],
+				classes[i].mi_owners[j], classes[i].mi_owners[j],
+				classes[i].mi_flags[j] );
+		printf("\n");
+	}
+	return 0;
+}
+
+
+put_cmd(ac, av)
+	int ac;
+	char **av;
+{
+	int cc;
+
+	pvm_packf("%+ %d", PvmDataDefault, 0);
+	cc = pvm_putinfo(av[1], pvm_getsbuf(), (int)atoi(av[2]));
+	if (cc < 0)
+		printf("%s\n", pvm_errlist[-cc]);
+	return 0;
+}
+
+
+export_cmd(ac, av)
+	int ac;
+	char **av;
+{
+	int i;
+	char *p;
+
+	if (ac == 1) {
+		p = getenv("PVM_EXPORT");
+		printf("PVM_EXPORT=%s\n", p ? p : "");
+
+	} else {
+		for (i = 1; i < ac; i++)
+			pvm_export(av[i]);
+	}
+	return 0;
+}
+
+
+unexport_cmd(ac, av)
+	int ac;
+	char **av;
+{
+	int i;
+
+	for (i = 1; i < ac; i++)
+		pvm_unexport(av[i]);
+	return 0;
+}
+
+
