@@ -1,12 +1,15 @@
 
+/* $Id: host.h,v 1.5 1999/07/08 19:00:20 kohl Exp $ */
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -31,7 +34,27 @@
  *
  *	Host table data structures.
  *
-$Log: host.h,v $
+ * $Log: host.h,v $
+ * Revision 1.5  1999/07/08 19:00:20  kohl
+ * Fixed "Log" keyword placement.
+ * 	- indent with " * " for new CVS.
+ *
+ * Revision 1.4  1997/06/25  22:08:45  pvmsrc
+ * Markus adds his frigging name to the author list of
+ * 	every file he ever looked at...
+ *
+ * Revision 1.3  1997/04/30  21:25:57  pvmsrc
+ * SGI Compiler Warning Cleanup.
+ *
+ * Revision 1.2  1997/01/28  19:27:56  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.1  1996/09/23  23:43:17  pvmsrc
+ * Initial revision
+ *
+ * Revision 1.4  1995/05/17  16:10:01  manchek
+ * added HF_OVERLOAD
+ *
  * Revision 1.3  1994/06/21  18:30:47  manchek
  * added HF_SPEED flag
  *
@@ -58,6 +81,7 @@ struct hostd {
 	char *hd_wdir;				/* pvmd working dir */
 	char *hd_sopts;				/* hoster options */
 	int hd_flag;
+	int hd_dsig;				/* data signature */
 	int hd_err;					/* error code */
 	int hd_mtu;					/* max snd/rcv length */
 	struct sockaddr_in hd_sad;	/* UDP address/port */
@@ -67,7 +91,7 @@ struct hostd {
 	struct pkt *hd_opq;			/* outstanding packets to host */
 	int hd_nop;					/* length of opq */
 	struct pkt *hd_rxq;			/* packet reordering queue from host */
-	struct mesg *hd_rxm;		/* to-us msg reassembly from host */
+	struct pmsg *hd_rxm;		/* to-us msg reassembly from host */
 	struct timeval hd_rtt;		/* estd round-trip time to host */
 	int hd_speed;				/* cpu relative speed */
 	struct mca *hd_mcas;		/* from-host mca cache */
@@ -79,6 +103,7 @@ struct hostd {
 /* for advisory host table */
 #define	HF_NOSTART	0x100		/* don't start pvmd */
 #define	HF_SPEED	0x200		/* hd_speed is set */
+#define	HF_OVERLOAD	0x400		/* allow overloaded (>1 pvmd) host */
 
 
 /* Host table */
@@ -94,17 +119,19 @@ struct htab {
 	struct hostd **ht_hosts;	/* hosts */
 };
 
-struct hostd *hd_new __ProtoGlarp__((int ind));
-void hd_free __ProtoGlarp__((struct hostd *hp));
-void hd_unref __ProtoGlarp__((struct hostd *hp));
+struct hostd *hd_new __ProtoGlarp__(( int ));
+void hd_free __ProtoGlarp__(( struct hostd * ));
+void hd_unref __ProtoGlarp__(( struct hostd * ));
+void hd_dump __ProtoGlarp__(( struct hostd * ));
 
-struct hostd *nametohost __ProtoGlarp__((struct htab *htp, char *name));
-struct hostd *indtohost __ProtoGlarp__((struct htab *htp, int tid));
-struct hostd *tidtohost __ProtoGlarp__((struct htab *htp, int tid));
-struct htab *ht_new __ProtoGlarp__((int len));
-int ht_insert __ProtoGlarp__((struct htab *htp, struct hostd *hp));
-int ht_delete __ProtoGlarp__((struct htab *htp, struct hostd *hp));
-void ht_free __ProtoGlarp__((struct htab *htp));
-struct htab *readhostfile __ProtoGlarp__((char *fn));
-int ht_merge __ProtoGlarp__((struct htab *dst, struct htab *src));
+struct hostd *nametohost __ProtoGlarp__(( struct htab *, char * ));
+struct hostd *indtohost __ProtoGlarp__(( struct htab *, int ));
+struct hostd *tidtohost __ProtoGlarp__(( struct htab *, int ));
+struct htab *ht_new __ProtoGlarp__(( int ));
+void ht_insert __ProtoGlarp__(( struct htab *, struct hostd * ));
+void ht_delete __ProtoGlarp__(( struct htab *, struct hostd * ));
+void ht_free __ProtoGlarp__(( struct htab * ));
+struct htab *readhostfile __ProtoGlarp__(( char * ));
+int ht_merge __ProtoGlarp__(( struct htab *, struct htab * ));
+void ht_dump __ProtoGlarp__(( struct htab * ));
 
