@@ -1,12 +1,16 @@
 
+static char rcsid[] =
+	"$Id: waitc.c,v 1.8 1999/07/08 19:00:18 kohl Exp $";
+
 /*
- *         PVM version 3.3:  Parallel Virtual Machine System
+ *         PVM version 3.4:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
- *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
- *                   (C) 1992 All Rights Reserved
+ *      Authors:  J. J. Dongarra, G. E. Fagg, M. Fischer
+ *          G. A. Geist, J. A. Kohl, R. J. Manchek, P. Mucci,
+ *         P. M. Papadopoulos, S. L. Scott, and V. S. Sunderam
+ *                   (C) 1997 All Rights Reserved
  *
  *                              NOTICE
  *
@@ -31,7 +35,50 @@
  *
  *	Wait context descriptors.
  *
-$Log: waitc.c,v $
+ * $Log: waitc.c,v $
+ * Revision 1.8  1999/07/08 19:00:18  kohl
+ * Fixed "Log" keyword placement.
+ * 	- indent with " * " for new CVS.
+ *
+ * Revision 1.7  1998/11/20  20:04:19  pvmsrc
+ * Changes so that win32 will compile & build. Also, common
+ * Changes so that compiles & builds on NT. Also
+ * common source on win32 & unix.
+ * (Spanker=sscott)
+ *
+ * Revision 1.6  1997/06/27  17:32:59  pvmsrc
+ * Updated for WIN32 header files & Authors.
+ *
+ * Revision 1.5  1997/03/06  21:50:24  pvmsrc
+ * Yanked out #includes for <netinet/in.h> and <netinet/tcp.h>.
+ * 	- dups with lpvm.h #includes...
+ *
+ * Revision 1.4  1997/01/28  19:27:52  pvmsrc
+ * New Copyright Notice & Authors.
+ *
+ * Revision 1.3  1996/10/25  13:58:10  pvmsrc
+ * Replaced old #includes for protocol headers:
+ * 	- <pvmsdpro.h>, "ddpro.h", "tdpro.h"
+ * With #include of new combined header:
+ * 	- <pvmproto.h>
+ *
+ * Revision 1.2  1996/10/24  21:18:31  pvmsrc
+ * Moved #include "global.h" to end of #include's for typing.
+ * Added #include of new "lpvm.h" in place of explicitly extern-ed
+ * 	globals...
+ *
+ * Revision 1.1  1996/09/23  23:44:47  pvmsrc
+ * Initial revision
+ *
+ * Revision 1.6  1995/11/02  16:36:44  manchek
+ * added NEEDSENDIAN switch
+ *
+ * Revision 1.5  1995/07/28  16:04:07  manchek
+ * switch endian includes on flag, not arch name
+ *
+ * Revision 1.4  1995/07/19  21:27:33  manchek
+ * use new function pvmnametag instead of [dts]mname
+ *
  * Revision 1.3  1994/07/18  19:25:18  manchek
  * wait_get() checks hostpart, ignoring TIDPVMD.
  * added create/free logging
@@ -45,35 +92,52 @@ $Log: waitc.c,v $
  */
 
 #include <stdio.h>
-#ifdef IMA_BSD386
+#ifdef NEEDMENDIAN
 #include <machine/endian.h>
 #endif
-#ifdef IMA_LINUX
+#ifdef NEEDENDIAN
 #include <endian.h>
 #endif
+#ifdef NEEDSENDIAN
+#include <sys/endian.h>
+#endif
+
+#include <pvm3.h>
+#ifdef WIN32
+#ifndef _XDR_HEADERS
+#define _XDR_HEADERS
+#include "..\xdr\types.h"
+#include "..\xdr\xdr.h"
+#endif
+#else
+#include <rpc/types.h>
+#include <rpc/xdr.h>
+#include <sys/socket.h>
+#include <sys/time.h>
+#endif
+
 #ifdef IMA_TITN
 #include <bsd/sys/types.h>
 #else
 #include <sys/types.h>
 #endif
-#include <sys/time.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
+
 #ifdef	SYSVSTR
 #include <string.h>
 #else
 #include <strings.h>
 #endif
-#include "global.h"
-#include "ddpro.h"
-#include "protoglarp.h"
+
+#include <pvmproto.h>
 #include "pvmalloc.h"
-#include "mesg.h"
-#include "host.h"
-#include "waitc.h"
+#include "pmsg.h"
 #include "listmac.h"
 #include "bfunc.h"
+#include "lpvm.h"
+#include "host.h"
+#include "waitc.h"
+#include "global.h"
+
 
 /***************
  **  Globals  **
@@ -81,14 +145,7 @@ $Log: waitc.c,v $
  ***************/
 
 extern void pvmbailout();
-char *dmname();
-
-extern int debugmask;				/* from pvmd.c */
-extern struct htab *hosts;			/* from pvmd.c */
-extern int myhostpart;				/* from pvmd.c */
-extern int tidlmask;				/* from pvmd.c */
-
-struct waitc *waitlist = 0;
+char *pvmnametag();
 
 
 /***************
@@ -96,13 +153,14 @@ struct waitc *waitlist = 0;
  **           **
  ***************/
 
-static char rcsid[] = "$Id: waitc.c,v 1.3 1994/07/18 19:25:18 manchek Exp $";
-static char pvmtxt[512];				/* scratch for error log */
+static int widbase = 0;
+static int widrange = ~0;
 static char *waitnames[] = {
 	"addhost", "spawn", "hoststart", "task",
 	"delhost", "htupd", "pstat", "taskx",
 	"mstat", "db", "hostf", "hosta", "hostsync",
-	"taskstart"
+	"taskstart",
+	"routea", "routed"
 };
 
 
@@ -127,9 +185,13 @@ waitkind(kind)
 */
 
 void
-wait_init()
+wait_init(base, range)
+	int base;			/* base value for wait context ids */
+	int range;			/* mask for id generation */
 {
 	if (!waitlist) {
+		widbase = base;
+		widrange = range;
 		waitlist = TALLOC(1, struct waitc, "wait");
 		BZERO((char*)waitlist, sizeof(struct waitc));
 		waitlist->wa_link = waitlist;
@@ -159,13 +221,13 @@ wait_new(kind)
 	* waitlist until we find a vacant slot.
 	*/
 
-	if (++lastwid > tidlmask)
+	if (++lastwid > widrange)
 		lastwid = 1;
 	startwid = lastwid;
 	wp = waitlist;
 
 	while (1) {
-		wid = myhostpart + lastwid;
+		wid = widbase + lastwid;
 		while (wp->wa_wid < wid)
 			if ((wp = wp->wa_link) == waitlist)
 				break;
@@ -173,12 +235,12 @@ wait_new(kind)
 		if (wp->wa_wid != wid)
 			break;
 
-		if (++lastwid > tidlmask) {
+		if (++lastwid > widrange) {
 			lastwid = 1;
 			wp = waitlist;
 		}
 		if (lastwid == startwid) {
-			pvmlogerror("wait_new() out of wids???\n");
+			pvmlogprintf("wait_new() out of wids\n");
 			pvmbailout(0);		/* XXX this is kinda harsh */
 		}
 	}
@@ -188,7 +250,7 @@ wait_new(kind)
 	*/
 
 	if (!(wp2 = TALLOC(1, struct waitc, "wait"))) {
-		pvmlogerror("wait_new() can't get memory\n");
+		pvmlogprintf("wait_new() can't get memory\n");
 		pvmbailout(0);
 	}
 	wp2->wa_wid = wid;
@@ -201,8 +263,8 @@ wait_new(kind)
 	wp2->wa_spec = 0;
 
 	LISTPUTBEFORE(wp, wp2, wa_link, wa_rlink);
-	if (debugmask & PDMWAITC) {
-		pvmlogerror("wait_new():\n");
+	if (pvmdebmask & PDMWAITC) {
+		pvmlogprintf("wait_new():\n");
 		wait_dump(wp2);
 	}
 	return wp2;
@@ -238,12 +300,12 @@ void
 wait_delete(wp)
 	struct waitc *wp;
 {
-	if (debugmask & PDMWAITC) {
-		pvmlogerror("wait_delete():\n");
+	if (pvmdebmask & PDMWAITC) {
+		pvmlogprintf("wait_delete():\n");
 		wait_dump(wp);
 	}
 	if (wp->wa_mesg)
-		mesg_unref(wp->wa_mesg);
+		pmsg_unref(wp->wa_mesg);
 
 	if (wp->wa_link) {
 		LISTDELETE(wp, wa_link, wa_rlink);
@@ -265,7 +327,7 @@ wait_delete(wp)
 struct waitc *
 wait_get(hp, mp, kind)
 	struct hostd *hp;
-	struct mesg *mp;
+	struct pmsg *mp;
 	int kind;
 {
 	struct waitc *wp;
@@ -273,21 +335,19 @@ wait_get(hp, mp, kind)
 	if (!mp->m_wid)
 		return 0;
 	if (!(wp = wait_find(mp->m_wid))) {
-		sprintf(pvmtxt, "waitc_get() cod %s from t%x wid %d not found\n",
-				dmname(mp->m_cod), mp->m_src, mp->m_wid);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("waitc_get() tag %s from t%x wid %d not found\n",
+				pvmnametag(mp->m_tag, (int *)0), mp->m_src, mp->m_wid);
 		return 0;
 	}
 	if (hp && hp->hd_hostpart != (wp->wa_on & TIDHOST)) {
-		sprintf(pvmtxt, "waitc_get() cod %s from t%x wid %d wrong host\n",
-				dmname(mp->m_cod), mp->m_src, mp->m_wid);
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("waitc_get() tag %s from t%x wid %d wrong host\n",
+				pvmnametag(mp->m_tag, (int *)0), mp->m_src, mp->m_wid);
 		return 0;
 	}
 	if (kind && kind != wp->wa_kind) {
-		sprintf(pvmtxt, "waitc_get() cod %s from t%x wid %d wrong kind: %s\n",
-				dmname(mp->m_cod), mp->m_src, mp->m_wid, waitkind(wp->wa_kind));
-		pvmlogerror(pvmtxt);
+		pvmlogprintf("waitc_get() tag %s from t%x wid %d wrong kind: %s\n",
+				pvmnametag(mp->m_tag, (int *)0), mp->m_src, mp->m_wid,
+				waitkind(wp->wa_kind));
 		return 0;
 	}
 	return wp;
@@ -298,26 +358,17 @@ void
 wait_dump(wp)
 	struct waitc *wp;
 {
-	struct hostd *hp;
 	struct waitc *wp2;
-	char *p = pvmtxt;
 
-	sprintf(p, " wid %d kind %s on ", wp->wa_wid, waitkind(wp->wa_kind));
-	p += strlen(p);
-	if (!TIDISTASK(wp->wa_on) && (hp = tidtohost(hosts, wp->wa_on)))
-		sprintf(p, "%s", hp->hd_name);
-	else
-		sprintf(p, "t%x", wp->wa_on);
-	p += strlen(p);
-	sprintf(p, " tid t%x dep %d peer {",
-		wp->wa_tid, wp->wa_dep);
-	for (wp2 = wp->wa_peer; wp2 != wp; wp2 = wp2->wa_peer) {
-		p += strlen(p);
-		sprintf(p, " %d", wp2->wa_wid);
-	}
-	p += strlen(p);
-	sprintf(p, " } cnt %d\n", wp->wa_count);
-	pvmlogerror(pvmtxt);
+	pvmlogprintf("wid %d kind %s on 0x%x tid 0x%x dep %d peer {",
+			wp->wa_wid,
+			waitkind(wp->wa_kind),
+			wp->wa_on,
+			wp->wa_tid,
+			wp->wa_dep);
+	for (wp2 = wp->wa_peer; wp2 != wp; wp2 = wp2->wa_peer)
+		pvmlogprintf(" %d", wp2->wa_wid);
+	pvmlogprintf(" } cnt %d\n", wp->wa_count);
 }
 
 
@@ -326,7 +377,7 @@ wait_dumpall()
 {
 	struct waitc *wp;
 
-	pvmlogerror("wait_dumpall()\n");
+	pvmlogprintf("wait_dumpall()\n");
 	for (wp = waitlist->wa_link; wp != waitlist; wp = wp->wa_link)
 		wait_dump(wp);
 }
